//
//  Base64Tests.m
//  CandyStore
//
//  Created by Daniel Norton on 8/3/11.
//  Copyright 2011 Daniel Norton. All rights reserved.
//

#import "Base64Tests.h"
#import "NSData+Base64.h"


NSString * const TestString = @"{"
	"\"signature\" = \"AjoiNUlqGhyTlyTQZzid+DudLMea7xwWRK4iMv6P3enPv8dp200k8nOov1H9UgOkJWlHe+R3+S/x"
	"pwlSbZp8SYGSrHjsmVocBtLoMGZTH+Pmqd13tb0TErRc6zUadekqQAIxhtsaL6Ojm2ELJ64RoaJUOGgfb2e7SXc5cBBeQw"
	"AHAAADVzCCA1MwggI7oAMCAQICCGUUkU3ZWAS1MA0GCSqGSIb3DQEBBQUAMH8xCzAJBgNVBAYTAlVTMRMwEQYDVQQKDApB"
	"cHBsZSBJbmMuMSYwJAYDVQQLDB1BcHBsZSBDZXJ0aWZpY2F0aW9uIEF1dGhvcml0eTEzMDEGA1UEAwwqQXBwbGUgaVR1bm"
	"VzIFN0b3JlIENlcnRpZmljYXRpb24gQXV0aG9yaXR5MB4XDTA5MDYxNTIyMDU1NloXDTE0MDYxNDIyMDU1NlowZDEjMCEG"
	"A1UEAwwaUHVyY2hhc2VSZWNlaXB0Q2VydGlmaWNhdGUxGzAZBgNVBAsMEkFwcGxlIGlUdW5lcyBTdG9yZTETMBEGA1UECg"
	"wKQXBwbGUgSW5jLjELMAkGA1UEBhMCVVMwgZ8wDQYJKoZIhvcNAQEBBQADgY0AMIGJAoGBAMrRjF2ct4IrSdiTChaI0g8p"
	"wv/cmHs8p/RwV/rt/91XKVhNl4XIBimKjQQNfgHsDs6yju++DrKJE7uKsphMddKYfFE5rGXsAdBEjBwRIxexTevx3HLEFG"
	"At1moKx509dhxtiIdDgJv2YaVs49B0uJvNdy6SMqNNLHsDLzDS9oZHAgMBAAGjcjBwMAwGA1UdEwEB/wQCMAAwHwYDVR0j"
	"BBgwFoAUNh3o4p2C0gEYtTJrDtdDC5FYQzowDgYDVR0PAQH/BAQDAgeAMB0GA1UdDgQWBBSpg4PyGUjFPhJXCBTMzaN+mV"
	"8k9TAQBgoqhkiG92NkBgUBBAIFADANBgkqhkiG9w0BAQUFAAOCAQEAEaSbPjtmN4C/IB3QEpK32RxacCDXdVXAeVReS5Fa"
	"Zxc+t88pQP93BiAxvdW/3eTSMGY5FbeAYL3etqP5gm8wrFojX0ikyVRStQ+/AQ0KEjtqB07kLs9QUe8czR8UGfdM1EumV/"
	"UgvDd4NwNYxLQMg4WTQfgkQQVy8GXZwVHgbE/UC6Y7053pGXBk51NPM3woxhd3gSRLvXj+loHsStcTEqe9pBDpmG5+sk4t"
	"w+GK3GMeEN5/+e1QT9np/Kl1nj+aBw7C0xsy0bFnaAd1cSS6xdory/CUvM6gtKsmnOOdqTesbp0bs8sn6Wqs0C9dgcxRHu"
	"OMZ2tm8npLUm7argOSzQ==\";"

	"\"purchase-info\" = \"ewoJIml0ZW0taWQiID0gIjQ1MzA5NDA5OCI7Cgkib3JpZ2luYWwtdHJhbnNhY3Rpb24taWQi"
	"ID0gIjEwMDAwMDAwMDQ2OTkyODkiOwoJInB1cmNoYXNlLWRhdGUiID0gIjIwMTEtMDgtMDMgMDY6Mzg6MzcgRXRjL0dNVC"
	"I7CgkicHJvZHVjdC1pZCIgPSAiY29tLmJyaW1zdGVhZC5jYW5keXN0b3JlLmJpZ2NhbmR5amFyIjsKCSJ0cmFuc2FjdGlv"
	"bi1pZCIgPSAiMTAwMDAwMDAwNDY5OTI4OSI7CgkicXVhbnRpdHkiID0gIjEiOwoJIm9yaWdpbmFsLXB1cmNoYXNlLWRhdG"
	"UiID0gIjIwMTEtMDgtMDMgMDY6Mzg6MzcgRXRjL0dNVCI7CgkiYmlkIiA9ICJjb20uYnJpbXN0ZWFkLkNhbmR5U3RvcmUi"
	"OwoJImJ2cnMiID0gIjEuMCI7Cn0=\";"

	"\"environment\" = \"Sandbox\";"
	"\"pod\" = \"100\";"
	"\"signing-status\" = \"0\";"
"}";

NSString * const TestEncoded = @"eyJzaWduYXR1cmUiID0gIkFqb2lOVWxxR2h5VGx5VFFaemlkK0R1ZExNZWE3eHdXUk"
	"s0aU12NlAzZW5QdjhkcDIwMGs4bk9vdjFIOVVnT2tKV2xIZStSMytTL3hwd2xTYlpwOFNZR1NySGpzbVZvY0J0TG9NR1pU"
	"SCtQbXFkMTN0YjBURXJSYzZ6VWFkZWtxUUFJeGh0c2FMNk9qbTJFTEo2NFJvYUpVT0dnZmIyZTdTWGM1Y0JCZVF3QUhBQU"
	"FEVnpDQ0ExTXdnZ0k3b0FNQ0FRSUNDR1VVa1UzWldBUzFNQTBHQ1NxR1NJYjNEUUVCQlFVQU1IOHhDekFKQmdOVkJBWVRB"
	"bFZUTVJNd0VRWURWUVFLREFwQmNIQnNaU0JKYm1NdU1TWXdKQVlEVlFRTERCMUJjSEJzWlNCRFpYSjBhV1pwWTJGMGFXOX"
	"VJRUYxZEdodmNtbDBlVEV6TURFR0ExVUVBd3dxUVhCd2JHVWdhVlIxYm1WeklGTjBiM0psSUVObGNuUnBabWxqWVhScGIy"
	"NGdRWFYwYUc5eWFYUjVNQjRYRFRBNU1EWXhOVEl5TURVMU5sb1hEVEUwTURZeE5ESXlNRFUxTmxvd1pERWpNQ0VHQTFVRU"
	"F3d2FVSFZ5WTJoaGMyVlNaV05sYVhCMFEyVnlkR2xtYVdOaGRHVXhHekFaQmdOVkJBc01Fa0Z3Y0d4bElHbFVkVzVsY3lC"
	"VGRHOXlaVEVUTUJFR0ExVUVDZ3dLUVhCd2JHVWdTVzVqTGpFTE1Ba0dBMVVFQmhNQ1ZWTXdnWjh3RFFZSktvWklodmNOQV"
	"FFQkJRQURnWTBBTUlHSkFvR0JBTXJSakYyY3Q0SXJTZGlUQ2hhSTBnOHB3di9jbUhzOHAvUndWL3J0LzkxWEtWaE5sNFhJ"
	"QmltS2pRUU5mZ0hzRHM2eWp1KytEcktKRTd1S3NwaE1kZEtZZkZFNXJHWHNBZEJFakJ3Ukl4ZXhUZXZ4M0hMRUZHQXQxbW"
	"9LeDUwOWRoeHRpSWREZ0p2MllhVnM0OUIwdUp2TmR5NlNNcU5OTEhzREx6RFM5b1pIQWdNQkFBR2pjakJ3TUF3R0ExVWRF"
	"d0VCL3dRQ01BQXdId1lEVlIwakJCZ3dGb0FVTmgzbzRwMkMwZ0VZdFRKckR0ZERDNUZZUXpvd0RnWURWUjBQQVFIL0JBUU"
	"RBZ2VBTUIwR0ExVWREZ1FXQkJTcGc0UHlHVWpGUGhKWENCVE16YU4rbVY4azlUQVFCZ29xaGtpRzkyTmtCZ1VCQkFJRkFE"
	"QU5CZ2txaGtpRzl3MEJBUVVGQUFPQ0FRRUFFYVNiUGp0bU40Qy9JQjNRRXBLMzJSeGFjQ0RYZFZYQWVWUmVTNUZhWnhjK3"
	"Q4OHBRUDkzQmlBeHZkVy8zZVRTTUdZNUZiZUFZTDNldHFQNWdtOHdyRm9qWDBpa3lWUlN0USsvQVEwS0VqdHFCMDdrTHM5"
	"UVVlOGN6UjhVR2ZkTTFFdW1WL1VndkRkNE53Tll4TFFNZzRXVFFmZ2tRUVZ5OEdYWndWSGdiRS9VQzZZNzA1M3BHWEJrNT"
	"FOUE0zd294aGQzZ1NSTHZYaitsb0hzU3RjVEVxZTlwQkRwbUc1K3NrNHR3K0dLM0dNZUVONS8rZTFRVDlucC9LbDFuaith"
	"Qnc3QzB4c3kwYkZuYUFkMWNTUzZ4ZG9yeS9DVXZNNmd0S3Ntbk9PZHFUZXNicDBiczhzbjZXcXMwQzlkZ2N4Ukh1T01aMn"
	"RtOG5wTFVtN2FyZ09TelE9PSI7InB1cmNoYXNlLWluZm8iID0gImV3b0pJbWwwWlcwdGFXUWlJRDBnSWpRMU16QTVOREE1"
	"T0NJN0Nna2liM0pwWjJsdVlXd3RkSEpoYm5OaFkzUnBiMjR0YVdRaUlEMGdJakV3TURBd01EQXdNRFEyT1RreU9Ea2lPd2"
	"9KSW5CMWNtTm9ZWE5sTFdSaGRHVWlJRDBnSWpJd01URXRNRGd0TURNZ01EWTZNemc2TXpjZ1JYUmpMMGROVkNJN0Nna2lj"
	"SEp2WkhWamRDMXBaQ0lnUFNBaVkyOXRMbUp5YVcxemRHVmhaQzVqWVc1a2VYTjBiM0psTG1KcFoyTmhibVI1YW1GeUlqc0"
	"tDU0owY21GdWMyRmpkR2x2YmkxcFpDSWdQU0FpTVRBd01EQXdNREF3TkRZNU9USTRPU0k3Q2draWNYVmhiblJwZEhraUlE"
	"MGdJakVpT3dvSkltOXlhV2RwYm1Gc0xYQjFjbU5vWVhObExXUmhkR1VpSUQwZ0lqSXdNVEV0TURndE1ETWdNRFk2TXpnNk"
	"16Y2dSWFJqTDBkTlZDSTdDZ2tpWW1sa0lpQTlJQ0pqYjIwdVluSnBiWE4wWldGa0xrTmhibVI1VTNSdmNtVWlPd29KSW1K"
	"MmNuTWlJRDBnSWpFdU1DSTdDbjA9IjsiZW52aXJvbm1lbnQiID0gIlNhbmRib3giOyJwb2QiID0gIjEwMCI7InNpZ25pbm"
	"ctc3RhdHVzIiA9ICIwIjt9";

@implementation Base64Tests

- (void)testShouldConvertToBase64 {
	
	NSData *data = [TestString dataUsingEncoding:[NSString defaultCStringEncoding]];
	NSString *encoded = [data base64EncodedString];
	STAssertEqualObjects(encoded, TestEncoded, nil);
}

- (void)testShouldConvertFromBase64 {
	
	NSData *data = [NSData dataFromBase64String:TestEncoded];
	NSString *unencoded = [[NSString alloc] initWithData:data encoding:[NSString defaultCStringEncoding]];
	STAssertEqualObjects(unencoded, TestString, nil);
}


@end

